--##################################################################
--######             START OF SCRIPT                          ######
--##################################################################
--##   Filename:  zmProcedure.sql
--##   Updated:   28 June 2023
--##   Purpose:   Create procedure to allow testing of SQL emulating
--##              how Sage X3 uses cursors
--##################################################################
--##
--##  Run this script by using the command
--##     exec zmRunCursorTest CUSTNAME_0
--##  (where 'CUSTNAME_0' is the partial name of the customer you want to search for)
--##    e.g. 
--##     exec zmRunCursorTest name1
--##
--##################################################################
--
--
CREATE or ALTER PROCEDURE zmRunCursorTest @mzCustName NVARCHAR
AS
BEGIN
DECLARE @mzN INTEGER
DECLARE @mzStartDate DATETIME
DECLARE @mzEndDate DATETIME
DECLARE @message NVARCHAR(MAX)
DECLARE @mzCustomerName NVARCHAR(MAX) 
--
PRINT '====    START  ======'
SET @mzStartDate = getdate()
PRINT 'Start time:' +  CONVERT(NVARCHAR,  @mzStartDate,  113);   
PRINT '==================='
set @mzN = 0
--
DECLARE @mzStatement NVARCHAR(MAX)
-- set @mzStatement = 'Select TBL_.ROWID, TBL_.* 
-- From SEED.ZMSQL TBL_ 
-- WHERE CUSTNAME_0 like ''%' + @mzCustomerName +  '%''
-- Option (FAST 1)'
set @mzStatement = 'Select TBL_.ROWID, TBL_.* 
From SEED.ZMSQL TBL_ 
WHERE CUSTNAME_0 like ''%name1%''
Option (FAST 1)'
DECLARE @statHandle INTEGER
DECLARE @curHandle INTEGER
DECLARE @scrollopt INTEGER = 0x10  -- FAST_FORWARD
DECLARE @ccopt INTEGER = 1   -- READ_ONLY
DECLARE @mzrowcount INTEGER 
DECLARE @mzrowfetchsize INTEGER = 20 -- Buffer Size
DECLARE @rc INTEGER
DECLARE @fetchtype INTEGER = 2 -- Next row
DECLARE @mzRownum INTEGER = 1
PRINT  '|Ready to go...|' + @mzStatement + '|'  + @mzCustomerName + '|'
--
exec sp_cursorprepexec @statHandle output,@curHandle output,NULL,@mzStatement,@scrollopt output,@ccopt output,@mzrowcount output
PRINT  '|CursorPrep|' + CAST(@statHandle as NVARCHAR(MAX)) +'|'+ CAST(@curHandle as NVARCHAR(MAX)) +'|'+CAST(@scrollopt as NVARCHAR(MAX)) +'|'+ CAST(@ccopt as NVARCHAR(MAX)) +'|'+ CAST(@mzrowcount as NVARCHAR(MAX)) +'|'
--
WHILE @@FETCH_STATUS = 0
BEGIN
	exec sp_cursorfetch @curHandle, @fetchtype, @mzN, @mzrowfetchsize
	PRINT  '|CursorFetch|' + CAST(@curHandle as NVARCHAR(MAX)) +'|'+ CAST( @fetchtype as NVARCHAR(MAX)) +'|'+CAST(@mzN as NVARCHAR(MAX))  +'|'+ CAST(@mzrowfetchsize as NVARCHAR(MAX)) +'|'
END
--
PRINT  '|Done!|'
exec sp_cursorclose @curHandle
exec sp_cursorunprepare @statHandle
---
--PRINT 'Number of iterations:' + CAST(@mzN as NVARCHAR(MAX)) + ' multipled by buffer size:' + CAST(@mzrowfetchsize as NVARCHAR(MAX))
PRINT '====    END  ======'
SET @mzEndDate = getdate()
PRINT 'End time:' +  CONVERT(NVARCHAR,  @mzEndDate,  113);
PRINT '==================='
END
---
--##################################################################
--######               END OF SCRIPT                          ######
--##################################################################
